/*
 * Program symulujący mysz przy pomocy joystika analogowego
 * podłączenie za pośrednictwem ESP32 BlueTooth LE 
 * v.1-04
 */
#include <BleConnectionStatus.h>
#include <BleMouse.h>

BleMouse bleMouse;
const int VRxPin = 34;
const int VRyPin = 35;
int potValueX = 0, potValueY = 0;
int centrum_PortX, centrum_PortY, licznik_usrednienia;
#define ILE_USREDNIEN 50
bool stat_centr;
int pauza;
int wektorX=0, wektorY=0;

static double przesuw_x_ulamek=0, przesuw_y_ulamek=0;


bool WyliczCentrumXY(int ValueX, int ValueY);
bool KonwersjaADC_przesuwKursora(int *p_pauza, int *p_wektorX, int *p_wektorY, int ValueX, int ValueY);
double ObliczPrzesuw(int przesuw,bool przesuw_plus,double przesuw_ulamek);

//double ObliczPrzesuw1(int przesuw,bool przesuw_plus,double przesuw_ulamek);

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  Serial.println("Starting BLE work!");
  bleMouse.begin();
  centrum_PortX = 0;
  centrum_PortY = 0;
  licznik_usrednienia = 0;
  delay(500);
}

void loop() {
  // put your main code here, to run repeatedly:

  potValueX = analogRead(VRxPin);
 //   Serial.print("VRx=");
 //   Serial.println(potValueX);

  potValueY = analogRead(VRyPin);
 //   Serial.print("VRy=");
 //   Serial.println(potValueY);

  stat_centr = WyliczCentrumXY(potValueX, potValueY);
  //  Serial.print("centrum XY=");
  //  Serial.print(centrum_PortX);
  //  Serial.print(", ");
  //  Serial.println(centrum_PortY);
  //  Serial.println(licznik_usrednienia,stat_centr);

  if (stat_centr==false)
  {
    pauza =10;
  }
  else
  {
    bool stat_poz = KonwersjaADC_przesuwKursora(&pauza, &wektorX, &wektorY, potValueX, potValueY);
  
    Serial.println("pauza, x, y, potX, potY");
    Serial.print(pauza);    Serial.print(" ;"); 
    Serial.print(wektorX);  Serial.print(" ;"); 
    Serial.print(wektorY);  Serial.print(" ;"); 
    Serial.print(potValueX);Serial.print(" ;"); 
    Serial.println(potValueY);  
  
  
    if (bleMouse.isConnected()) 
    {
            bleMouse.move(wektorY,wektorX);
    }
  }

  delay(pauza);
}

/*
   wyliczanie centrum X,Y poprzez uśrednione pomiary
   we: wartości odczytane z przetwornika dla portów X i Y joystika
   wy: true -centra XY wyliczone
*/
bool WyliczCentrumXY(int ValueX, int ValueY)      
{
  if (licznik_usrednienia >= ILE_USREDNIEN) return true;

  centrum_PortX += ValueX;
  centrum_PortY += ValueY;
  licznik_usrednienia++;
  if (licznik_usrednienia >= ILE_USREDNIEN)
  {
    centrum_PortX = centrum_PortX / licznik_usrednienia;
    centrum_PortY = centrum_PortY / licznik_usrednienia;
    return true;
  } else return false;
}

/*
   konwersja wartości ADC X i Y na przesunięcie kursora
   wy: true -ruch kursora
*/
bool KonwersjaADC_przesuwKursora(int *p_pauza, int *p_wektorX, int *p_wektorY, int ValueX, int ValueY)
{
#define PAUZA_MS_MAX      50
#define PAUZA_MS_MIN      10
#define CENTRUM_ZERO_ADC  300

  bool przesuw_plus_x, przesuw_plus_y;
  int przesuw_x, przesuw_y;
  double wspolcz;
 
  //wyznaczanie kierunku ruchu kursora w osi X i Y
  przesuw_x = centrum_PortX - ValueX;
  if (przesuw_x < 0) przesuw_plus_x = true;//odwrócenie kierunku przesuwu w osi X
  else przesuw_plus_x               = false;//dla joystika analogowego typu Thumb Slide Joystick - SparkFun COM-09426
  przesuw_y = centrum_PortY - ValueY;
  if (przesuw_y < 0) przesuw_plus_y = false;
  else przesuw_plus_y               = true;

  //określenie czy pozycja absolutna przesunięć obu wektorów znajduje się w obszarze centrum
  przesuw_x = abs(przesuw_x);
  przesuw_y = abs(przesuw_y);
  if ((przesuw_x <= CENTRUM_ZERO_ADC) && (przesuw_y <= CENTRUM_ZERO_ADC))
  {
    //pozycja joystika w strefie zerowego ruchu
    *p_pauza = 50;
    *p_wektorX = 0;
    *p_wektorY = 0;
    przesuw_x_ulamek=0;
    przesuw_y_ulamek=0;
    return false;
  }
  else
  {
    if (przesuw_x>CENTRUM_ZERO_ADC)
    {
/*      
 *    to procedura symulacji dla testów
 *    if (przesuw_plus_x==false) *p_wektorX =-1 *2;
 *    else *p_wektorX =1 *2;
*/
      przesuw_x_ulamek =ObliczPrzesuw(przesuw_x,przesuw_plus_x,przesuw_x_ulamek);
      *p_wektorX =przesuw_x_ulamek;
      przesuw_x_ulamek-=*p_wektorX;
    }
    else *p_wektorX =0;

    if (przesuw_y>CENTRUM_ZERO_ADC)
    {
/*
 *  to procedura symulacji dla testów
 *  if (przesuw_plus_y==false) *p_wektorY =-1 *2;
 *  else *p_wektorY =1 *2;
 */
      przesuw_y_ulamek =ObliczPrzesuw(przesuw_y,przesuw_plus_y,przesuw_y_ulamek);
      *p_wektorY =przesuw_y_ulamek;
      przesuw_y_ulamek-=*p_wektorY;
    }
    else *p_wektorY =0;

   *p_pauza =1;
   return true;
  }


}
/*
 * obliczanie przesuwu w funkcji wychylenia joystika
 */
double ObliczPrzesuw(int przesuw,bool przesuw_plus,double przesuw_ulamek)
{
const int delta_stopien_tab[]={CENTRUM_ZERO_ADC+300,CENTRUM_ZERO_ADC+700,CENTRUM_ZERO_ADC+800,CENTRUM_ZERO_ADC+1000};
const double delta_mnoznik_tab[]={0.33,0.66,1,1.5,2.0};
const char ile_delta=4;
char x;  
double ulamek;
  
    for (x=0;x<ile_delta;x++)
    {
      if (przesuw <delta_stopien_tab[x]) break;
    }
    ulamek =delta_mnoznik_tab[x];
    if (przesuw_plus==false) ulamek =0-ulamek;
    
    return przesuw_ulamek+ulamek;
}

//double ObliczPrzesuw1(int przesuw,bool przesuw_plus,double przesuw_ulamek)
//{
//double ulamek;
//#define ZAKRES_ADC_MAX  1000
//#define MNOZNIK_PRZESUW_MIN 0.33
//#define MNOZNIK_PRZESUW_MAX 1.5
//
//    ulamek =(((double)(przesuw) /(CENTRUM_ZERO_ADC+ZAKRES_ADC_MAX)) *MNOZNIK_PRZESUW_MAX)+MNOZNIK_PRZESUW_MIN;
//    if (przesuw_plus==false) ulamek =0-ulamek;
//    
//    return przesuw_ulamek+ulamek;
//  
//  return 0;
//}
